import { ContentArea } from '@c/ui/ContentArea';
import { useQuery, } from '@tanstack/react-query';
import { Button, Col, Divider, Row, message } from 'antd';
import React, {FC, useCallback, useRef, useState} from 'react';
import 'twin.macro';
import Logo from 'assets/logo.png'
import { reportTenementDetail } from '@q/publicity';
import { isCorrectResult } from '@u/asyncs';
import Card from './components/Card';
import Unit from './components/Unit';
import { outputWithPrecision } from '@u/funcs';
import html2pdf from "html2pdf.js";
import dayjs from "dayjs";
import TopTitle from './components/TopTitle';
import TopContent from './components/TopContent';
// 核心代码
import { useSearchParams } from 'react-router-dom'
import {getRoundNumber} from "@/utils";

function getAmount(amount: string, loss: string, refund: string, price: string): string {
    return `${
        getRoundNumber(
            Number(
                getRoundNumber(
                    Number(
                        (Number(
                                (Number(amount) + Number(loss)
                                )
                            ) -
                            Number(refund)
                        )
                    )
                )
            ) * Number(price)
        )}`
}


export const UserReport: FC = () => {
    const [searchParams] = useSearchParams()
    const tid = searchParams.get("tenement");
    const rid = searchParams.get("report");
    const amount = useRef<string>();
    const { data } = useQuery(
        ['report-tenement-detail', rid, tid],
        () => reportTenementDetail(rid, tid),
        {
            select: res => {
                if (isCorrectResult(res)) {
                    amount.current = res.amount;
                    return res.detail
                }
                message.error(res.message || '获取商户电费详情失败');
                return null;
            }
        }
    );

    const [downloadLoading, setDownloadLoading] = useState(false);

    const printRef = useRef()

    const handlePrint = async () => {
        const opt = {
            margin: 0,
            filename: `${data?.comprehensive?.startDate ? dayjs(data?.comprehensive?.startDate).format("YYYY年MM月") : null}${data?.tenement?.fullName || '报表'}.pdf`,
            image: { type: "jpeg", quality: 0.98, },
            enableLinks: true,
            html2canvas: { scale: 3, useCORS: true, allowTaint: false,  },
            jsPDF: {
                unit: 'pt', // pt、mm、cm、in
                format: 'a4',
                orientation: 'p' // 纵向p，横向l

            },
        };
        setDownloadLoading(true)
        // eslint-disable-next-line @typescript-eslint/no-unsafe-call
        await html2pdf(printRef.current, opt);
        setDownloadLoading(false);
    };


    const renderTable = useCallback(() => {
        return data?.meters?.map((item, index) => {
            const finalAmount = Number(item?.overall?.amount || 0) + Number(item?.loss?.amount || 0) + Number(item?.publicAmount || 0)
            return (
                <tr key={index}>
                    <td>{item?.meterNo}</td>
                    <td>{item?.address}</td>
                    <td>{outputWithPrecision(item?.startNumber, 2, '-')}</td>
                    <td>{outputWithPrecision(item?.endNumber, 2, '-')}</td>
                    <td>{outputWithPrecision(item?.displayRatio, 2, '-')}</td>
                    <td>{outputWithPrecision(item?.overall?.amount, 2, '-')}</td>
                    {/* <td>{outputWithPrecision(item?.finalTotal ? (1 / (1 - Number(item?.loss?.amount) / Number(item?.finalTotal)) - 1) * 100 : undefined, 2, '-')} %</td> */}
                    <td>{outputWithPrecision(item?.loss?.amount, 2, '-')}</td>
                    <td>{outputWithPrecision(item?.publicAmount, 2, '-')}</td>
                    <td>{outputWithPrecision(finalAmount, 2, '-')}</td>
                </tr>
                )
        })
    }, [data?.meters])

  return (
      <>
          <style>
              {
                  `
                        td {
                            padding: 0.2cm
                        }
                        tr td:nth-child(1) {
                            width: 3.5cm;
                        }
                        tr td:nth-child(2) {
                            width: 4.5cm;
                        }
                        tr td:nth-child(3), tr td:nth-child(4), tr td:nth-child(5) {
                            width: 1.8cm;
                        }
                        tr td:nth-child(6), tr td:nth-child(7), tr td:nth-child(8), tr td:nth-child(9) {
                            width: 2cm;
                        }
                        #content, .ant-col {
                            font-size: 0.34cm;
                        }
                    `
              }
          </style>
          <div style={{
              position: 'fixed',
              width: '100%',
              height: '100%',
              left: 0,
              right: 0,
              top: 0,
              bottom: 0,
              margin: 'auto',
              zIndex: 99,
              background: '#fff',
              display: 'flex',
              justifyContent: 'center',
              alignItems: "center"
          }}>

              <Button loading={downloadLoading} type='primary' tw='' onClick={() => handlePrint()}> 下载 </Button>


          </div>
          <div ref={printRef}>
              <ContentArea>
                  <div id="content">
                      <div>
                          <img src={Logo} alt="" width={60} height={60}/>
                      </div>
                      <div tw='mt-4' style={{fontSize: "18px", fontWeight: 600}}>
                          尊敬的{data?.tenement?.fullName}，这是贵公司 &nbsp;
                          {dayjs(data?.comprehensive?.startDate).format("MM")} 月的电费账单
                      </div>
                      <div tw='flex mt-6 flex-row' style={{backgroundColor: "#f0f0f0", padding: '18px 20px'}}>
                          <div tw='flex flex-1 flex-col'>
                              <Row gutter={16}>
                                  <Col span={5} style={{display: "flex", flexDirection: "column"}}>
                                      <div>账单周期</div>
                                      <div style={{
                                          fontSize: "20px",
                                          fontWeight: 600,
                                          flex: 1,
                                          display: "flex",
                                          alignItems: "center"
                                      }}>
                                          {data?.comprehensive?.startDate ? dayjs(data?.comprehensive?.startDate).format("YYYY年MM月") : null}
                                      </div>
                                      {/* <div> {data?.comprehensive?.startDate} - {data?.comprehensive?.endDate} </div> */}
                                  </Col>
                                  <Col span={19}>
                                      <Row gutter={16}>
                                          <Col span={6}> <TopTitle> 商户编号 </TopTitle>
                                              <TopContent> {data?.tenement?.id} </TopContent> </Col>
                                          <Col span={6}>
                                              <TopTitle>商户名称</TopTitle><TopContent>{data?.tenement?.fullName}</TopContent>
                                          </Col>
                                          <Col span={6}> <TopTitle>用户类型</TopTitle><TopContent>工商业</TopContent>
                                          </Col>
                                          <Col span={6}> <TopTitle>电压等级</TopTitle><TopContent>0.4kv</TopContent>
                                          </Col>
                                      </Row>
                                      <Divider dashed tw='mt-2 mb-2 flex flex-col w-full'/>
                                      <Row gutter={16}>
                                          <div tw='mt-3 flex-1 flex flex-row'>
                                              <Col span={6}> <TopTitle>所属园区</TopTitle>
                                                  <TopContent> {data?.park?.name} </TopContent> </Col>
                                              <Col span={6}> <TopTitle>用户地址</TopTitle>
                                                  <TopContent> {data?.tenement?.address} </TopContent> </Col>
                                              <Col span={6}>
                                                  <TopTitle>抄表日期</TopTitle><TopContent>{data?.comprehensive?.endDate}</TopContent>
                                              </Col>
                                              <Col span={6}>
                                                  <TopTitle>购电方式</TopTitle><TopContent>市场化零售客户</TopContent>
                                              </Col>
                                          </div>
                                      </Row>
                                  </Col>
                              </Row>
                          </div>
                      </div>
                      <div>
                          <div tw='mt-6'>
                              账单信息
                              <Divider tw='mt-2 mb-2 flex flex-col w-full'/>
                          </div>
                          <div tw='mt-2'>
                              <div tw='flex flex-row'>
                                  <Card>
                                      <h4 tw='mt-1 mb-1'> 合计电量： </h4>
                                      <div> {
                                          outputWithPrecision(
                                              `${Number(amount.current || 0)}`, 2, '-')} &nbsp;&nbsp;千瓦时
                                      </div>
                                  </Card>
                              </div>
                              <div tw='mt-3'>
                                  电费构成
                                  <Divider tw='mt-2 mb-4 flex flex-col w-full'/>
                              </div>
                              {
                                  data?.park?.meter04kvType === 0 ? (
                                      <div tw='flex flex-row items-center' style={{
                                          height: 'fit-content',
                                          alignItems: "stretch",
                                          flexGrow: "stratch"
                                      }}>
                                          <div tw='flex flex-row items-center flex-wrap'>
                                              <Unit> （ </Unit>
                                              <Card>
                                                  <h4 tw='mt-1 mb-1'> 电度电量： </h4>
                                                  <div> {outputWithPrecision(data?.comprehensive?.consumption, 2, '-')} &nbsp;&nbsp;千瓦时</div>
                                              </Card>
                                              {
                                                  data?.tenement?.feeType === 3 ? null : (
                                                      <>
                                                          <Unit> + </Unit>
                                                          <Card>
                                                              <h4 tw='mt-1 mb-1'> 线损电量： </h4>
                                                              <div> {outputWithPrecision(data?.comprehensive?.lossAmount, 2, '-')} &nbsp;&nbsp;千瓦时</div>
                                                          </Card>
                                                      </>
                                                  )
                                              }

                                              <Unit> + </Unit>
                                              <Card>
                                                  <h4 tw='mt-1 mb-1'> 公摊电量： </h4>
                                                  <div> {outputWithPrecision(data?.comprehensive?.publicAmount, 2, '-')} &nbsp;&nbsp;千瓦时</div>
                                              </Card>
                                              <Unit> - </Unit>
                                              <Card>
                                                  <h4 tw='mt-1 mb-1'> 退补电量： </h4>
                                                  <div> {outputWithPrecision(data?.comprehensive?.publicAmount, 2, '-')} &nbsp;&nbsp;千瓦时</div>
                                              </Card>
                                              <Unit> ） </Unit><Unit> * </Unit>
                                              <Card>
                                                  <h4 tw='mt-1 mb-1'> 电度单价： </h4>
                                                  <div> {outputWithPrecision(data?.comprehensive?.price, 6, '-')} &nbsp;&nbsp;千瓦时/元</div>
                                              </Card>
                                              <Unit> + </Unit>
                                              <Card>
                                                  <h4 tw='mt-1 mb-1'> 基本电费： </h4>
                                                  <div> {outputWithPrecision(data?.comprehensive?.basicPooled, 2, '-')} </div>
                                              </Card>
                                              <Unit> + </Unit>
                                              <Card>
                                                  <h4 tw='mt-1 mb-1'> 调整电费： </h4>
                                                  <div> {outputWithPrecision(data?.comprehensive?.adjustPooled, 2, '-')} </div>
                                              </Card>
                                          </div>
                                          <div tw='flex flex-row items-center' style={{height: '100%', flex: 1}}>
                                              <Unit> = </Unit>
                                              <Card style={{height: 200}}>
                                                  <h4 tw='mt-1 mb-1'> 本期总电费（元）： </h4>
                                                  <div> {outputWithPrecision(data?.comprehensive?.total, 2, '-')} </div>
                                              </Card>
                                          </div>
                                      </div>
                                  ) : (
                                      <div tw='' style={{
                                          height: 'fit-content',
                                          alignItems: "stretch",
                                          flexGrow: "stratch"
                                      }}>
                                          <div tw='flex flex-row items-center flex-wrap'>
                                              <Unit> （ </Unit>
                                              <Card>
                                                  <h4 tw='mt-1 mb-1'> 尖电量： </h4>
                                                  <div> {outputWithPrecision(data?.comprehensive?.sharpAmount, 2, '-')} &nbsp;&nbsp;千瓦时</div>
                                              </Card>
                                              {
                                                  data?.tenement?.feeType === 3 ? null : (
                                                      <>
                                                          <Unit> + </Unit>
                                                          <Card>
                                                              <h4 tw='mt-1 mb-1'> 尖线损电量： </h4>
                                                              <div> {outputWithPrecision(data?.comprehensive?.lossSharp, 2, '-')} &nbsp;&nbsp;千瓦时</div>
                                                          </Card>
                                                      </>
                                                  )
                                              }

                                              <Unit> + </Unit>
                                              <Card>
                                                  <h4 tw='mt-1 mb-1'> 尖公摊电量： </h4>
                                                  <div> {outputWithPrecision(data?.comprehensive?.poolSharp || '0', 2, '-')} &nbsp;&nbsp;千瓦时</div>
                                              </Card>
                                              <Unit> - </Unit>
                                              <Card>
                                                  <h4 tw='mt-1 mb-1'> 尖退补电量： </h4>
                                                  <div> {outputWithPrecision(data?.comprehensive?.refundSharp, 2, '-')} &nbsp;&nbsp;千瓦时</div>
                                              </Card>
                                              <Unit> ） </Unit><Unit> * </Unit>
                                              <Card>
                                                  <h4 tw='mt-1 mb-1'> 尖用电单价： </h4>
                                                  <div> {outputWithPrecision(data?.comprehensive?.priceSharp, 6, '-')} &nbsp;&nbsp;千瓦时/元</div>
                                              </Card>
                                              <Unit> = </Unit>
                                              <Card>
                                                  <h4 tw='mt-1 mb-1'> 尖电费： </h4>
                                                  <div> {outputWithPrecision(getAmount(
                                                      data?.comprehensive?.sharpAmount || "0",
                                                      data?.tenement?.feeType === 3 ? "0" : (data?.comprehensive?.lossSharp || "0"),
                                                      data?.comprehensive?.refundSharp || "0",
                                                      data?.comprehensive?.priceSharp || "0",
                                                  ), 2, '-')} &nbsp;&nbsp;</div>
                                              </Card>
                                          </div>
                                          <div tw='flex flex-row items-center flex-wrap'>
                                              <Unit> （ </Unit>
                                              <Card>
                                                  <h4 tw='mt-1 mb-1'> 峰电量： </h4>
                                                  <div> {outputWithPrecision(data?.comprehensive?.peakAmount, 2, '-')} &nbsp;&nbsp;千瓦时</div>
                                              </Card>
                                              {
                                                  data?.tenement?.feeType === 3 ? null : (
                                                      <>
                                                          <Unit> + </Unit>
                                                          <Card>
                                                              <h4 tw='mt-1 mb-1'> 峰线损电量： </h4>
                                                              <div> {outputWithPrecision(data?.comprehensive?.lossPeak, 2, '-')} &nbsp;&nbsp;千瓦时</div>
                                                          </Card>
                                                      </>
                                                  )
                                              }

                                              <Unit> + </Unit>
                                              <Card>
                                                  <h4 tw='mt-1 mb-1'> 峰公摊电量： </h4>
                                                  <div> {outputWithPrecision(data?.comprehensive?.poolPeak, 2, '-')} &nbsp;&nbsp;千瓦时</div>
                                              </Card>
                                              <Unit> - </Unit>
                                              <Card>
                                                  <h4 tw='mt-1 mb-1'> 峰退补电量： </h4>
                                                  <div> {outputWithPrecision(data?.comprehensive?.refundPeak, 2, '-')} &nbsp;&nbsp;千瓦时</div>
                                              </Card>
                                              <Unit> ） </Unit><Unit> * </Unit>
                                              <Card>
                                                  <h4 tw='mt-1 mb-1'> 峰用电单价： </h4>
                                                  <div> {outputWithPrecision(data?.comprehensive?.pricePeak, 6, '-')} &nbsp;&nbsp;千瓦时/元</div>
                                              </Card>
                                              <Unit> = </Unit>
                                              <Card>
                                                  <h4 tw='mt-1 mb-1'> 峰电费： </h4>
                                                  <div> {outputWithPrecision(getAmount(
                                                      data?.comprehensive?.peakAmount || "0",
                                                      data?.tenement?.feeType === 3 ? "0" : (data?.comprehensive?.lossPeak || "0"),
                                                      data?.comprehensive?.refundPeak || "0",
                                                      data?.comprehensive?.pricePeak || "0",
                                                  ), 2, '-')} &nbsp;&nbsp;</div>
                                              </Card>
                                          </div>
                                          <div tw='flex flex-row items-center flex-wrap'>
                                              <Unit> （ </Unit>
                                              <Card>
                                                  <h4 tw='mt-1 mb-1'> 平电量： </h4>
                                                  <div> {outputWithPrecision(data?.comprehensive?.flatAmount, 2, '-')} &nbsp;&nbsp;千瓦时</div>
                                              </Card>
                                              {
                                                  data?.tenement?.feeType === 3 ? null : (
                                                      <>
                                                          <Unit> + </Unit>
                                                          <Card>
                                                              <h4 tw='mt-1 mb-1'> 平线损电量： </h4>
                                                              <div> {outputWithPrecision(data?.comprehensive?.lossFlat, 2, '-')} &nbsp;&nbsp;千瓦时</div>
                                                          </Card>
                                                      </>
                                                  )
                                              }

                                              <Unit> + </Unit>
                                              <Card>
                                                  <h4 tw='mt-1 mb-1'> 平公摊电量： </h4>
                                                  <div> {outputWithPrecision(data?.comprehensive?.poolFlat, 2, '-')} &nbsp;&nbsp;千瓦时</div>
                                              </Card>
                                              <Unit> - </Unit>
                                              <Card>
                                                  <h4 tw='mt-1 mb-1'> 平退补电量： </h4>
                                                  <div> {outputWithPrecision(data?.comprehensive?.refundFlat, 2, '-')} &nbsp;&nbsp;千瓦时</div>
                                              </Card>
                                              <Unit> ） </Unit><Unit> * </Unit>
                                              <Card>
                                                  <h4 tw='mt-1 mb-1'> 平用电单价： </h4>
                                                  <div> {outputWithPrecision(data?.comprehensive?.priceFlat, 6, '-')} &nbsp;&nbsp;千瓦时/元</div>
                                              </Card>
                                              <Unit> = </Unit>
                                              <Card>
                                                  <h4 tw='mt-1 mb-1'> 平电费： </h4>
                                                  <div> {outputWithPrecision(getAmount(
                                                      data?.comprehensive?.flatAmount || "0",
                                                      data?.tenement?.feeType === 3 ? "0" : (data?.comprehensive?.lossFlat || "0"),
                                                      data?.comprehensive?.refundFlat || "0",
                                                      data?.comprehensive?.priceFlat || "0",
                                                  ), 2, '-')} &nbsp;&nbsp;</div>
                                              </Card>
                                          </div>
                                          <div tw='flex flex-row items-center flex-wrap'>
                                              <Unit> （ </Unit>
                                              <Card>
                                                  <h4 tw='mt-1 mb-1'> 谷电量： </h4>
                                                  <div> {outputWithPrecision(data?.comprehensive?.valleyAmount, 2, '-')} &nbsp;&nbsp;千瓦时</div>
                                              </Card>
                                              {
                                                  data?.tenement?.feeType === 3 ? null : (
                                                      <>
                                                          <Unit> + </Unit>
                                                          <Card>
                                                              <h4 tw='mt-1 mb-1'> 谷线损电量： </h4>
                                                              <div> {outputWithPrecision(data?.comprehensive?.lossValley, 2, '-')} &nbsp;&nbsp;千瓦时</div>
                                                          </Card>
                                                      </>
                                                  )
                                              }

                                              <Unit> + </Unit>
                                              <Card>
                                                  <h4 tw='mt-1 mb-1'> 谷公摊电量： </h4>
                                                  <div> {outputWithPrecision(data?.comprehensive?.poolValley, 2, '-')} &nbsp;&nbsp;千瓦时</div>
                                              </Card>
                                              <Unit> - </Unit>
                                              <Card>
                                                  <h4 tw='mt-1 mb-1'> 谷退补电量： </h4>
                                                  <div> {outputWithPrecision(data?.comprehensive?.refundValley, 2, '-')} &nbsp;&nbsp;千瓦时</div>
                                              </Card>
                                              <Unit> ） </Unit><Unit> * </Unit>
                                              <Card>
                                                  <h4 tw='mt-1 mb-1'> 谷用电单价： </h4>
                                                  <div> {outputWithPrecision(data?.comprehensive?.priceValley, 6, '-')} &nbsp;&nbsp;千瓦时/元</div>
                                              </Card>
                                              <Unit> = </Unit>
                                              <Card>
                                                  <h4 tw='mt-1 mb-1'> 谷电费： </h4>
                                                  <div> {outputWithPrecision(getAmount(
                                                      data?.comprehensive?.valleyAmount || "0",
                                                      data?.tenement?.feeType === 3 ? "0" : (data?.comprehensive?.lossValley || "0"),
                                                      data?.comprehensive?.refundValley || "0",
                                                      data?.comprehensive?.priceValley || "0",
                                                  ), 2, '-')} &nbsp;&nbsp;</div>
                                              </Card>
                                          </div>
                                          <div tw='flex flex-row items-center flex-wrap'>
                                              <Card>
                                                  <h4 tw='mt-1 mb-1'> 尖电费： </h4>
                                                  <div> {outputWithPrecision(getAmount(
                                                      data?.comprehensive?.sharpAmount || "0",
                                                      data?.tenement?.feeType === 3 ? "0" : (data?.comprehensive?.lossSharp || "0"),
                                                      data?.comprehensive?.refundSharp || "0",
                                                      data?.comprehensive?.priceSharp || "0",
                                                  ), 2, '-')} </div>
                                              </Card>
                                              <Unit> + </Unit>
                                              <Card>
                                                  <h4 tw='mt-1 mb-1'> 峰电费： </h4>
                                                  <div> {outputWithPrecision(getAmount(
                                                      data?.comprehensive?.peakAmount || "0",
                                                      data?.tenement?.feeType === 3 ? "0" : (data?.comprehensive?.lossPeak || "0"),
                                                      data?.comprehensive?.refundPeak || "0",
                                                      data?.comprehensive?.pricePeak || "0",
                                                  ), 2, '-')} </div>
                                              </Card>
                                              <Unit> + </Unit>
                                              <Card>
                                                  <h4 tw='mt-1 mb-1'> 平电费： </h4>
                                                  <div> {outputWithPrecision(getAmount(
                                                      data?.comprehensive?.flatAmount || "0",
                                                      data?.tenement?.feeType === 3 ? "0" : (data?.comprehensive?.lossFlat || "0"),
                                                      data?.comprehensive?.refundFlat || "0",
                                                      data?.comprehensive?.priceFlat || "0",
                                                  ), 2, '-')}</div>
                                              </Card>
                                              <Unit> + </Unit>
                                              <Card>
                                                  <h4 tw='mt-1 mb-1'> 谷电费： </h4>
                                                  <div> {outputWithPrecision(getAmount(
                                                      data?.comprehensive?.valleyAmount || "0",
                                                      data?.tenement?.feeType === 3 ? "0" : (data?.comprehensive?.lossValley || "0"),
                                                      data?.comprehensive?.refundValley || "0",
                                                      data?.comprehensive?.priceValley || "0",
                                                  ), 2, '-')}</div>
                                              </Card>
                                              <Unit> = </Unit>
                                              <Card>
                                                  <h4 tw='mt-1 mb-1'> 本期总电费： </h4>
                                                  <div> {outputWithPrecision(data?.comprehensive?.total, 2, '-')} &nbsp;&nbsp;</div>
                                              </Card>
                                          </div>
                                          {/*<div tw='flex flex-row items-center' style={{height: '100%', flex: 1}}>*/}
                                          {/*    <Unit> = </Unit>*/}
                                          {/*    <Card style={{height: 200}}>*/}
                                          {/*        <h4 tw='mt-1 mb-1'> 本期总电费（元）： </h4>*/}
                                          {/*        <div> {outputWithPrecision(data?.comprehensive?.total, 2, '-')} </div>*/}
                                          {/*    </Card>*/}
                                          {/*</div>*/}
                                      </div>
                                  )
                              }
                          </div>
                          <div>
                              <div tw='mt-4'>
                                  用电明细
                                  <Divider tw='mt-2 mb-4 flex flex-col w-full'/>
                              </div>
                              <div tw="mt-3" style={{fontSize: 12}}>
                                  <table border={1} width={"100%"} cellSpacing="0">
                                      <thead>
                                      <tr>
                                          <td>电表编号</td>
                                          <td>电表地址</td>
                                          <td>起码</td>
                                          <td>止码</td>
                                          <td>表倍率</td>
                                          <td>电度电量</td>
                                          {/* <td>线损占比</td> */}
                                          {
                                              data?.tenement?.feeType === 3 ? null : (
                                                  <td>线损电量</td>
                                              )
                                          }
                                          <td>退补电量</td>
                                          <td>公摊电量</td>
                                          <td>合计电量</td>
                                      </tr>
                                      </thead>
                                      <tbody>
                                      {renderTable()}
                                      </tbody>
                                  </table>

                              </div>
                          </div>
                      </div>
                  </div>

              </ContentArea>
          </div>


      </>

  );
};
